@@ -1,21 +1,23 @@
 #' @title Problems of distribution of delay in deterministic projects
 #' @description This function calculates the delay of a project once it has been completed. In addition, it also calculates the distribution of the delay between the different activities with the proportional, truncated proportional and Shapley rule.
 #' @param duration Vector with the expected duration for each activity.
-#' @param precedence A matrix that indicates the order of precedence between activities. If the value \eqn{(i,j)} is equal to 1 then \eqn{i} precedes \eqn{j}.
+#' @param prec1and2 A matrix indicating the order of precedence type 1 and 2 between the activities (Default=matrix(0)). If value \eqn{(i,j)=1} then activity \eqn{i} precedes type \eqn{1} to \eqn{j}, and if \eqn{(i,j)=2} then activity \eqn{i} precedes type \eqn{2} to \eqn{j}. Cycles cannot exist in a project,  i.e. if an activity \eqn{i} precedes \eqn{j} then \eqn{j} cannot precede \eqn{i}.
+#' @param prec3and4 A matrix indicating the order of precedence type 3 and 4 between the activities (Default=matrix(0)). If value \eqn{(i,j)=3} then activity \eqn{i} precedes type \eqn{3} to \eqn{j}, and if \eqn{(i,j)=4} then activity \eqn{i} precedes type \eqn{4} to \eqn{j}. Cycles cannot exist in a project,  i.e. if an activity \eqn{i} precedes \eqn{j} then \eqn{j} cannot precede \eqn{i}.
 #' @param observed.duration Vector with the observed duration for each activity.
-#' @param delta Value to indicate the maximun time that the project can take without delay. If this value is not added, the function will use as delta the expected project time.
+#' @param delta Value to indicate the maximun time that the project can take without delay. If this is not added, the function will use as delta the expected project time. This value is only used if the function uses the default cost function.
+#' @param cost.function Delay costs function. If this value is not added, a default cost function will be used.
 #' @export
-#' @details Given a problem of sharing delays in a project \eqn{(N,\prec,\{\bar{X}_i\}_{i\in N},\{x_i\}_{i\in N})},  such that \eqn{\{\bar{X}_i\}_{i\in N}} is the expected value of activities' duration and \eqn{\{x_i\}_{i\in N}} the observed value. If \eqn{D(N,\prec,\{\bar{X}_i\}_{i\in N})} is the expected project time and \eqn{D(N,\prec,\{x_i\}_{i\in N})} is the observed project time, it has to \eqn{d=D(N,\prec,\{\bar{X}_i\}_{i\in N})-\delta>0} is the delay, where \eqn{\delta} can be any arbitrary value greater than zero. The following rules distribute the delay among the different activities.
+#' @details Given a problem of sharing delays in a project \eqn{(N,\prec,\{\bar{X}_i\}_{i\in N},\{x_i\}_{i\in N})},  such that \eqn{\{\bar{X}_i\}_{i\in N}} is the expected value of activities' duration and \eqn{\{x_i\}_{i\in N}} the observed value. If \eqn{D(N,\prec,\{\bar{X}_i\}_{i\in N})} is the expected project time and \eqn{D(N,\prec,\{x_i\}_{i\in N})} is the observed project time, it has to \eqn{d=D(N,\prec,\{\bar{X}_i\}_{i\in N})-\delta} is the delay, where \eqn{\delta} can be any arbitrary value greater than zero. The following rules distribute the delay costs among the different activities.
 #'
 #' The proportional rule, from \cite{Brânzei et al. (2002)}, distributes the delay, \eqn{d}, proportionally. So that each activity receives a payment of:
-#' \deqn{\phi_i=\frac{\displaystyle x_{i}-\bar{X}_{i}}{\displaystyle \sum_{j\in N}\max\{x_{j}-\bar{X}_{j},0\}}\cdot d \qquad if  \qquad x_{i}-\bar{X}_{i}>0}
-#' and 0  in another case.
+#' \deqn{\phi_i=\frac{\displaystyle x_{i}-\bar{X}_{i}}{\displaystyle \sum_{j\in N}\max\{x_{j}-\bar{X}_{j},0\}}\cdot C(D(N,\prec,\{\bar{X}_i\}_{i\in N})).}
 #'
 #' The truncated proportional rule, from \cite{Brânzei et al. (2002)}, distributes the delay, \eqn{d}, proportionally, where the individual delay of each player is reduced to \eqn{d} if if is larger. So that each activity receives a payment of:
-#' \deqn{\bar{\phi}_i=\frac{\displaystyle \min\{x_{i}-\bar{X}_{i},d\}}{\displaystyle \sum_{j\in N} \max\{\min\{x_{j}-\bar{X}_{j},d\},0\}}\cdot d \qquad if  \qquad x_{i}-\bar{X}_{i}>0}
-#' and 0  in another case.
+#' \deqn{\bar{\phi}_i=\frac{\displaystyle \min\{x_{i}-\bar{X}_{i},C(D(N,\prec,\{\bar{X}_i\}_{i\in N}))\}}{\displaystyle \sum_{j\in N} \max\{\min\{x_{j}-\bar{X}_{j},C(D(N,\prec,\{\bar{X}_i\}_{i\in N}))\},0\}}\cdot C(D(N,\prec,\{\bar{X}_i\}_{i\in N})).}
 #'
-#' Shapley rule distributes the delay, \eqn{d}, based on the Shapley value for TU games, see \cite{Bergantiños et al. (2018)}. Given a project problem with delays \eqn{(N,\prec,\{\bar{X}_i\}_{i\in N},\{x_i\}_{i\in N})}, its associated TU game, \eqn{(N,v)}, is \eqn{v(S)=\max\{D(N,\prec,(\{\bar{X}_i\}_{i\in N\backslash S},\{x_i\}_{i\in S}))-\delta,0\}} for all \eqn{S\subseteq N}. If the number of activities is greater than nine, the Shapley value, of the game \eqn{(N,v)}, is estimated using a unique sampling process for all players, see \cite{Castro et al. (2009)}.
+#'These values are only well defined when the sum of the individual delays is different from zero.
+#'
+#' Shapley rule distributes the delay, \eqn{d}, based on the Shapley value for TU games, see \cite{Bergantiños et al. (2018)}. Given a project problem with delays \eqn{(N,\prec,\{\bar{X}_i\}_{i\in N},\{x_i\}_{i\in N})}, its associated TU game, \eqn{(N,v)}, is \eqn{v(S)=C(D(N,\prec,(\{\bar{X}_i\}_{i\in N\backslash S},\{x_i\}_{i\in S})))} for all \eqn{S\subseteq N}, where \eqn{C} is the costs function (by default \eqn{C(D(N,\prec,y))=D(N,\prec,y)-\delta}. If the number of activities is greater than ten, the Shapley value, of the game \eqn{(N,v)}, is estimated using a unique sampling process for all players, see \cite{Castro et al. (2009)}.
 #' @references
 #' \describe{
 #'   \item{}{Bergantiños, G., Valencia-Toledo, A., & Vidal-Puga, J. (2018). Hart and Mas-Colell consistency in PERT problems. Discrete Applied Mathematics, 243, 11-20.}
@@ -25,24 +27,25 @@
 #' @return  The delay value and a solution matrix.
 #' @examples
 
-#' precedence<-matrix(c(0,1,0,1,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0),nrow=5,ncol=5,byrow=TRUE)
+#' prec1and2<-matrix(c(0,1,0,1,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0),nrow=5,ncol=5,byrow=TRUE)
 #' duration<-c(2,1,1,4,2)
 #' observed.duration<-c(2.5,1.25,2,4.5,3)
-#' delta<-6.5
-#' delay.pert(duration,precedence,observed.duration,delta)
+#' delta<-6
+#' delay.pert(duration,prec1and2=prec1and2,observed.duration=observed.duration,
+#' delta=delta,cost.function=NULL)
 
 
-delay.pert<-function(duration,precedence,observed.duration,delta=NULL){
+delay.pert<-function(duration,prec1and2=matrix(0),prec3and4=matrix(0),observed.duration,delta=NULL,cost.function=NULL){
 
-  or1<-order(organize(precedence)$Order[,2])
-  or2<-organize(precedence)$Order[,2]
-  precedence<-organize(precedence)$Precedence
+  or1<-order(organize(prec1and2,prec3and4)$Order[,2])
+  or2<-organize(prec1and2,prec3and4)$Order[,2]
+  precedence<-organize(prec1and2,prec3and4)$Precedence
   duration<-duration[or2]
   observed.duration<-observed.duration[or2]
   activities<-1:length(duration)
   n<-length(activities)
   SD<-numeric(length(activities))
-
+  w<-numeric(2^n-1)
   tiempo.early<-rep(0,n)
 
   ii<-as.logical(colSums(precedence))
@@ -70,10 +73,20 @@ delay.pert<-function(duration,precedence,observed.duration,delta=NULL){
 
   tiempo.observado<-max(tiempo.early+observed.duration)
 
-  if(tiempo.observado>tiempo){
-    cat("There has been a delay of = ", tiempo.observado-tiempo, "\n")
-    Prop<-((pmax(observed.duration-duration,0))/sum(pmax(observed.duration-duration,0)))*(tiempo.observado-tiempo)
-    TProp<-(pmin(pmax(observed.duration-duration,0),tiempo.observado-tiempo)/sum(pmin(pmax(observed.duration-duration,0),tiempo.observado-tiempo)))*(tiempo.observado-tiempo)
+
+
+    {
+    if(is.null(cost.function)==FALSE){
+      cat("There has been a delay of = ", cost.function(tiempo.observado), "\n")
+      Prop<-((observed.duration-duration)/sum(observed.duration-duration))*(cost.function(tiempo.observado))
+      TProp<-(pmin(observed.duration-duration,cost.function(tiempo.observado))/sum(pmin(observed.duration-duration,cost.function(tiempo.observado))))*(cost.function(tiempo.observado))
+    }
+    else{
+      cat("There has been a delay of = ", tiempo.observado-tiempo, "\n")
+    Prop<-((observed.duration-duration)/sum(observed.duration-duration))*(tiempo.observado-tiempo)
+    TProp<-(pmin(observed.duration-duration,tiempo.observado-tiempo)/sum(pmax(observed.duration-duration,tiempo.observado-tiempo)))*(tiempo.observado-tiempo)
+    }
+    }
 
     for(j in 1:length(activities)){
       duracion1<-duration
@@ -84,12 +97,12 @@ delay.pert<-function(duration,precedence,observed.duration,delta=NULL){
       }
       }
 
-
+      if(is.null(cost.function)==FALSE){w[j]<-cost.function(max(tiempo.early+duracion1))}
       SD[j]<-max(tiempo.early+duracion1)
     }
 
     {
-      if(length(activities)>9){
+      if(length(activities)>10){
         cat("shapley need some time to compute a", n, "player game \n")
         cat("Continue calculation of the Shapley value based on sampling? (Y = Yes, N = No) \n")
         continue <- scan(what = "character", n = 1)
@@ -106,8 +119,16 @@ delay.pert<-function(duration,precedence,observed.duration,delta=NULL){
                 for(j in 1:nn) {
                   tiempo.early[iii[j]]=max(tiempo.early[prec[j,]]+duracion1[prec[j,]])
                 }
-}
-                tiempo2[i]<-max(max(tiempo.early+duracion1)-tiempo,0)
+                }
+                {
+                  if(is.null(cost.function)==FALSE){
+                    tiempo2[i]<-cost.function(max(tiempo.early+duracion1))
+                  }
+                  else{
+                    tiempo2[i]<-max(tiempo.early+duracion1)-tiempo
+                  }
+                }
+
             }
             sh[x[1]]<-sh[x[1]]+tiempo2[1]
             sh[x[2:n]]<-sh[x[2:n]]+tiempo2[2:n]-tiempo2[1:(n-1)]
@@ -133,21 +154,31 @@ delay.pert<-function(duration,precedence,observed.duration,delta=NULL){
               tiempo.early[iii[i]]=max(tiempo.early[prec[i,]]+duracion1[prec[i,]])
             }
             }
+            if(is.null(cost.function)==FALSE){w[p]<-cost.function(max(tiempo.early+duracion1))}
             v[p]<-max(tiempo.early+duracion1)
             p<-p+1
           }
         }
 
-        v<-pmax(v-tiempo,0)
+        v<-v-tiempo
 
         #sh<-c(shapleyValue(n,v=v)$value)
 
-        z<-as.matrix(DefineGame(n,v)$Lex)
-        z<-as.vector(z)
-        coalitions <- set.func(c(0, z))
-        sh <- Shapley.value(coalitions)
-
 
+        {
+        if(is.null(cost.function)==FALSE){
+          z<-as.matrix(DefineGame(n,w)$Lex)
+          z<-as.vector(z)
+          coalitions <- set.func(c(0, z))
+          sh <- Shapley.value(coalitions)
+        }
+          else{
+            z<-as.matrix(DefineGame(n,v)$Lex)
+            z<-as.vector(z)
+            coalitions <- set.func(c(0, z))
+            sh <- Shapley.value(coalitions)
+          }
+        }
         }
     }
 
@@ -168,10 +199,6 @@ delay.pert<-function(duration,precedence,observed.duration,delta=NULL){
         return(round(A,5))
       }
     }
-  }
-  else{
-    cat("There is no delay", "\n")
-  }
 
 }
 
